COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	Copyright (c) GeoWorks 1993 -- All Rights Reserved

PROJECT:	
MODULE:		
FILE:		cidserCardServices.asm

AUTHOR:		Adam de Boor, Sep 29, 1993

ROUTINES:
	Name			Description
	----			-----------

	
REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Adam	9/29/93		Initial revision


DESCRIPTION:
	Interface to Card Services
		

	$Id: cidserCardServices.asm,v 1.1.1.4 93/11/15 23:11:18 adam Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

Resident	segment	resource


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		CIDSerCardServicesCallback
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Callback routine for Card Services events

CALLED BY:	Card Services

PASS:		al	-> function
		cx	-> socket
		dx	-> info
		di	-> 1st word in RegisterClient
		ds	-> dgroup (2nd word in RegisterClient)
		si	-> 3rd word in RegisterClient
		ss:bp	-> MTDRequest
		es:bx	-> buffer
		bx	-> Misc (when no buffer returned)

RETURN:		ax	<- status to return
		carry set on error,
		carry clear on success.

DESTROYED:	nothing

SIDE EFFECTS:
		None

PSEUDO CODE/STRATEGY:
		Notify registered client.

		See if a thread is spawned.

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	TS	7/ 7/93    	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
CIDSerCardServicesCallback	proc	far
	uses	cx, dx
	.enter

	cmp	al, CSEC_CARD_INSERTION
	je	doInsertion

	cmp	al, CSEC_REGISTRATION_COMPLETE
	jne	handleEvent
	
	mov	ds:[amRegistered], TRUE
	jmp	success

handleEvent:	
	mov	di, ax
	andnf	di, 0xff
	shl	di
	cmp	di, endEventRoutineTable - eventRoutineTable
	ja	unsupported
	jmp	cs:[eventRoutineTable][di-2]

	;--------------------
doExclusiveReq:
	call	CIDSerCSHandleExclusiveRequest
	jmp	done

	;--------------------
doInsertion:
	call	CIDSerIHandleInsertion
	jmp	success

	;--------------------
doRemoval:
	call	CIDSerRHandleRemoval
	jmp	success
	
	;--------------------
doInfo:
	test	es:[bx].CSGCIA_attributes, mask CSGCIAA_INFO_SUBFUNCTION
	jnz	unsupported	; only handle function 0
	
	mov	cx, cs:[clientInfo].CSGCIA_infoLen
	cmp	cx, es:[bx].CSGCIA_maxLen
	jbe	copyInfo
	mov	cx, es:[bx].CSGCIA_maxLen
copyInfo:
	segmov	ds, cs
	mov	si, offset clientInfo.CSGCIA_infoLen
	lea	di, es:[bx].CSGCIA_infoLen
	sub	cx, offset CSGCIA_infoLen	; not copying stuff up to here
	rep	movsb
	jmp	success

	;--------------------
unsupported:
	mov	ax, CSRC_UNSUPPORTED_FUNCTION
	stc
	jmp	done

	;--------------------
doIgnore:
success:
	mov	ax, CSRC_SUCCESS
	clc
done:
	.leave
	ret

clientInfo	CSGetClientInfoArgs <
	0,			; CSGCIA_maxLen
	size clientInfo,
	mask CSGCIAA_EXCLUSIVE_CARDS or \
		mask CSGCIAA_SHARABLE_CARDS or \
		mask CSGCIAA_MEMORY_CLIENT_DEVICE_DRIVER,
	<			; CSGCIA_clientInfo
		0100h,				; CSCI_revision
		0201h,				; CSCI_csLevel
		<
			29,				; CSDI_YEAR
			9,				; CSDI_MONTH
			22				; CSDI_DAY
		>,				; CSCI_revDate
		clientInfoName - clientInfo,	; CSCI_nameOffset
		length clientInfoName,		; CSCI_nameLength
		vendorString - clientInfo,	; CSCI_vStringOffset
		length vendorString		; CSCI_vStringLength
	>
>
	org	clientInfo.CSGCIA_clientInfo.CSCI_data
clientInfoName	char	"GEOS Serial Driver", 0
vendorString	char	"Geoworks", 0

DefCSEvent	macro	event, handler
	.assert ($-eventRoutineTable)/2 eq (event-1)
	nptr.near	handler
		endm
eventRoutineTable	label	nptr
DefCSEvent	CSEC_PM_BATTERY_DEAD,	doIgnore	; Battery Dead
DefCSEvent	CSEC_PM_BATTERY_LOW,	doIgnore	; Battery Low
DefCSEvent	CSEC_CARD_LOCK,		doIgnore	; Card Locked
DefCSEvent	CSEC_CARD_READY,	doIgnore	; Card Ready
DefCSEvent	CSEC_CARD_REMOVAL,	doRemoval	; Card Removal
DefCSEvent	CSEC_CARD_UNLOCK,	doIgnore	; Card Unlocked
DefCSEvent	CSEC_EJECTION_COMPLETE,	doIgnore	; Ejection Complete
DefCSEvent	CSEC_EJECTION_REQUEST,	doIgnore	; Ejection Request
DefCSEvent	CSEC_INSERTION_COMPLETE,doIgnore	; Insertion Complete
DefCSEvent	CSEC_INSERTION_REQUEST,	doIgnore	; Insertion Request
DefCSEvent	CSEC_PM_RESUME,		doIgnore	; Power Manager Resume
DefCSEvent	CSEC_PM_SUSPEND,	doIgnore	; Power Manager Suspend
DefCSEvent	CSEC_EXCLUSIVE_COMPLETE,doIgnore	; Exclusive Complete
DefCSEvent	CSEC_EXCLUSIVE_REQUEST,	doExclusiveReq	; Exclusive Request
DefCSEvent	CSEC_RESET_PHYSICAL,	doIgnore	; Reset Physical
DefCSEvent	CSEC_RESET_REQUEST,	doIgnore	; Reset Request
DefCSEvent	CSEC_CARD_RESET,	doIgnore	; Card Reset
DefCSEvent	CSEC_MTD_REQUEST,	unsupported	; MTD Request
DefCSEvent	CSEC_RESERVED_1,	unsupported	; UNDEFINED
DefCSEvent	CSEC_CLIENT_INFO,	doInfo		; Get Client Info
DefCSEvent	CSEC_TIMER_EXPIRED,	doIgnore	; Timer Expired
DefCSEvent	CSEC_SS_UPDATED,	doIgnore	; SS Updated
endEventRoutineTable	label	nptr

CIDSerCardServicesCallback	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		CIDSerCSHandleExclusiveRequest
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Cope with someone else wanting exclusive access to the card.

CALLED BY:	(INTERNAL) CIDSerCardServicesCallback
PASS:		ds	= dgroup
		cx	= socket
RETURN:		carry set if we don't want to allow it:
			ax	= CSRC_IN_USE
		carry clear if we're ok with it:
			ax	= CSRC_SUCCESS
DESTROYED:	nothing
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	10/26/93	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
CIDSerCSHandleExclusiveRequest proc	near
		uses	bx, di
		.enter
		call	CIDSerCSCheckCardInUse
		jnc	done
	;
	; Port is open, so don't allow the exclusive access.
	; 
		mov	ax, CSRC_IN_USE
		stc
done:
		.leave
		ret
CIDSerCSHandleExclusiveRequest endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		CIDSerCSCheckCardInUse
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	See if the card is in-use

CALLED BY:	(EXTERNAL) CIDSerCSHandleExclusiveRequest,
			   CIDSerRObjectionResolved
PASS:		cx	= socket number
		ds	= dgroup
RETURN:		carry set if port in-use
			ax	= destroyed
		carry clear if port not in-use
			ax	= CSRC_SUCCESS
DESTROYED:	bx, di
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	11/ 2/93	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
CIDSerCSCheckCardInUse proc	far
		.enter
		call	CIDSerUDerefSocket
		tst	ds:[bx].CIDSSI_support
		jz	ok
	;
	; Now see if the thing is open.
	; 
		mov	bx, ds:[bx].CIDSSI_port
		mov	di, DR_SERIAL_STAT_PORT
		call	ds:[serialStrat]
		jc	ok		; => port doesn't exist (?!)

		tst	ah
		jz	ok		; => not open
done:
		.leave
		ret
ok:
			CheckHack <CSRC_SUCCESS eq 0>
		clr	ax		; (clears carry)
		jmp	done
CIDSerCSCheckCardInUse endp

Resident	ends
