COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	Copyright (c) GeoWorks 1993 -- All Rights Reserved

PROJECT:	
MODULE:		
FILE:		cidserInsert.asm

AUTHOR:		Adam de Boor, Sep 29, 1993

ROUTINES:
	Name			Description
	----			-----------

	
REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Adam	9/29/93		Initial revision


DESCRIPTION:
	Functions to cope with card insertion.
		

	$Id: cidserInsert.asm,v 1.1.1.3 93/10/28 20:31:48 adam Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

Resident	segment	resource


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		CIDSerIHandleInsertion
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	See if the inserted card is one we like.

CALLED BY:	(EXTERNAL) CIDSerCardServicesCallback
PASS:		cx	= socket
		dx	= info
		ds	= dgroup
RETURN:		carry set on error, clear on success
		ax	= CardServicesReturnCode
DESTROYED:	nothing
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	9/29/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
CIDSerIHandleInsertion proc	near
		uses	bx, dx
		.enter
		call	CIDSerUDerefSocket
	;
	; Talk to Card ID to see what kind of card the thing is.
	; 
		clr	dx		; don't want string
		call	CIDGetCardType
		jc	setNo
		
		cmp	ax, PCMCT_SERIAL_CARD
		jne	setNo
setYes::
		mov	ds:[bx].CIDSSI_support, CIDSCS_YES

		tst	ds:[bx].CIDSSI_conflict
		jz	clearConflict
	;
	; Card was removed under protest -- reestablish the port parameters to
	; what they were before the thing was removed.
	; 
		push	bx
		mov	bx, ds:[bx].CIDSSI_port
		mov	di, DR_SERIAL_REESTABLISH_STATE
		call	ds:[serialStrat]
		pop	bx
clearConflict:
		mov	ds:[bx].CIDSSI_conflict, 0
done:
		.leave
		ret
setNo:
		mov	ds:[bx].CIDSSI_support, CIDSCS_NO
		jmp	done

CIDSerIHandleInsertion endp

Resident	ends

ProcessCode	segment	resource


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		CIDSerICheckSocket
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Check the passed socket to see if we support it, and do
		all the other fun things we have to do when we do support
		it.

CALLED BY:	DR_PCMCIA_CHECK_SOCKET
PASS:		cx	= socket number
		ds	= dgroup
RETURN:		carry set if card supported
DESTROYED:	di
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		if fixed data says we don't support it, we don't support it

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	9/29/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
CIDSerICheckSocket proc	far
socket		local	word	push cx
configArgs	local	CSGetConfigurationInfoArgs
		uses	ax, bx, cx, dx, si
		.enter

waitForRegistrationLoop:
		tst	ds:[amRegistered]
		jz	waitForRegistrationLoop

		call	CIDSerUDerefSocket
			CheckHack <CIDSCS_NO eq 0>
		tst	ds:[bx].CIDSSI_support
		jnz	processIt
fail:
		clc		; not a serial card
done:
		.leave
		ret

processIt:
	;
	; Fetch the configuration for the thing and tell the serial driver
	; about it.
	; 
		push	bx
		mov	ss:[configArgs].CSGCfIA_socket, cx
		segmov	es, ss
		lea	bx, ss:[configArgs]
		mov	cx, size configArgs
		CallCS	CSF_GET_CONFIGURATION_INFO
		jc	popBXFail
		
		mov	ax, ss:[configArgs].CSGCfIA_basePort1
		mov	cl, ss:[configArgs].CSGCfIA_assignedIRQ
		pop	bx
		push	bx
		mov	ds:[bx].CIDSSI_base, ax
		mov	ds:[bx].CIDSSI_irq, cl

		mov	bx, ss:[configArgs].CSGCfIA_socket
		mov	di, DR_SERIAL_DEFINE_PORT
		call	ds:[serialStrat]
		jc	popBXFail
		mov_tr	ax, bx
		pop	bx
	;
	; That succeeded. Remember the port number for later calls.
	; 
		mov	ds:[bx].CIDSSI_port, ax
	;
	; Register with the library, finally.
	; bx	<- geode handle
	; cx	<- socket
	; dx	<- cs handle
	; es:di	<- CSRegisterClientArgs
	; ax:si <- cs callback
	; 
		mov	bx, vseg regArgList
		call	MemLockFixedOrMovable
		mov	es, ax
		mov	di, offset regArgList

		mov	bx, handle 0

		mov	cx, ss:[socket]

		segmov	ds, dgroup, ax
		mov	dx, ds:[csHandle]

		mov	ax, segment CIDSerCardServicesCallback
		mov	si, offset CIDSerCardServicesCallback
		call	PCMCIARegisterDriver

		mov	bx, vseg regArgList
		call	MemUnlockFixedOrMovable

		stc			; happy happy happy
		jmp	done

popBXFail:
		pop	bx
		mov	ds:[bx].CIDSSI_support, CIDSCS_NO
		jmp	fail
CIDSerICheckSocket endp

ProcessCode	ends
