COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	Copyright (c) GeoWorks 1993 -- All Rights Reserved

PROJECT:	
MODULE:		
FILE:		cidserRemove.asm

AUTHOR:		Adam de Boor, Sep 30, 1993

ROUTINES:
	Name			Description
	----			-----------

	
REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Adam	9/30/93		Initial revision


DESCRIPTION:
	Stuff for coping with the removal of a card.
		

	$Id: cidserRemove.asm,v 1.1.1.4 93/11/15 23:11:37 adam Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

Resident	segment resource


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		CIDSerRHandleRemoval
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	See if the removed card is one we need

CALLED BY:	(EXTERNAL) CIDSerCardServicesCallback
PASS:		cx	= socket
		dx	= info
		ds	= dgroup
RETURN:		carry set on error, clear on success
		ax	= CardServicesReturnCode
DESTROYED:	di
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	9/29/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
CIDSerRHandleRemoval proc	near
		uses	bx, dx, bp
		.enter
		call	CIDSerUDerefSocket
	;
	; If card not supported, we're happy to see it go (why are we here?)
	; 
		tst	ds:[bx].CIDSSI_support
		jz	resetSupport

	;
	; Tell the serial driver ASAP that the port is gone.
	; 
		push	bx
		mov	bx, ds:[bx].CIDSSI_port
		mov	di, DR_SERIAL_PORT_ABSENT
		call	ds:[serialStrat]
	;
	; Now see if the thing is open.
	; 
		mov	di, DR_SERIAL_STAT_PORT
		call	ds:[serialStrat]
		pop	bx
		jc	resetSupport	; => doesn't exist (?!)
		tst	ah
		jz	resetSupport	; => not open, so who cares?
	;
	; It is open, so mark the port conflicted and tell the PCMCIA library
	; of our objections.
	; 
		mov	ds:[bx].CIDSSI_conflict, TRUE
		
		mov	dx, TRUE	; card may NOT be removed
		mov	bp, handle 0
		call	PCMCIAObjectToRemoval
resetSupport:
		mov	ds:[bx].CIDSSI_support, CIDSCS_NO
		clc
		mov	ax, CSRC_SUCCESS
		.leave
		ret
CIDSerRHandleRemoval endp

Resident	ends

ProcessCode	segment	resource


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		CIDSerRObjectionResolved
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Follow the dictates of our user.

CALLED BY:	DR_PCMCIA_OBJECTION_RESOLVED
PASS:		ds	= dgroup
		cx	= socket number
		dx	= PCMCIAObjectionResolution
RETURN:		carry set if told to clean up but can't
		carry clear if all ok.
DESTROYED:	nothing
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	9/30/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
CIDSerRObjectionResolved proc	far
		.enter
			CheckHack <PCMOR_CLEAN_UP eq 0>
		tst_clc	dx
		jz	attemptCleanUp
	;
	; in theory, since the removal was canceled and the card is back, we'd
	; release any access blocks we might have placed, allowing the card to
	; be reached again.... we currently set no blocks, though, so this
	; is a nop.
	;  
done:
		.leave
		ret
attemptCleanUp:
	;
	; Pretend success unless port is in-use.
	; 
		call	CIDSerCSCheckCardInUse
		jmp	done
CIDSerRObjectionResolved endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		CIDSerRCloseSocket
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Delete the port

CALLED BY:	DR_PCMCIA_CLOSE_SOCKET
PASS:		cx	= socket number
RETURN:		nothing
DESTROYED:	nothing
SIDE EFFECTS:	port gets deleted

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	9/30/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
CIDSerRCloseSocket proc	far
		uses	ax, di, ds, bx, cx
		.enter
		call	CIDSerUDerefSocket
		segmov	ds, dgroup, ax
		mov	ax, ds:[bx].CIDSSI_base
		mov	cl, SERIAL_PORT_DOES_NOT_EXIST	; turn off port
		mov	bx, -1		; not PCMCIA
		mov	di, DR_SERIAL_DEFINE_PORT
		call	ds:[serialStrat]
		.leave
		ret
CIDSerRCloseSocket endp


ProcessCode	ends
