COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	Copyright (c) GeoWorks 1993 -- All Rights Reserved

PROJECT:	PCMCIA Support
MODULE:		Common Code
FILE:		pcmEnumCIS.asm

AUTHOR:		Adam de Boor, May 18, 1993

ROUTINES:
	Name			Description
	----			-----------

	
REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Adam	5/18/93		Initial revision


DESCRIPTION:
	Functions to enumerate the contents of the CIS for the current card.
		

	$Id: pcmEnumCIS.asm,v 1.1.1.5 93/08/12 19:35:37 brianc Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

Init		segment	resource


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PCMEnumCIS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Call a callback function with each tuple from the CIS
		in turn.

CALLED BY:	(EXTERNAL)
PASS:		cx:dx	= callback function
			  PASS:		es:di	= tuple data
			  		cx	= # data bytes
					al	= CISTuple
			  RETURN:	carry set to stop processing
			  DESTROYED:	es, cx, dx, ax, di, si

		es:di	= 256-byte buffer into which to copy each tuple.
RETURN:		carry set if no CIS or no card present.
			ah	= SocketReturnCode
DESTROYED:	al, cx, dx, di
SIDE EFFECTS:	the 0th window is given to the 0th socket, and the 0th
		page is mapped to various and sundry

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/18/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
PCMEnumCIS		proc	far
callback	local	fptr.far	push cx, dx
buffer		local	fptr		push es, di
window		local	WindowData	; description of window #0
winSeg		local	sptr		; segment to which its page is mapped
lastBase	local	dword		; offset currently mapped to page
lastSpace	local	PageState	;  and the space from which it comes
longLinkSpace	local	PageState	; link to next CIS
longLinkBase	local	dword		;  if any
seenTuple	local	byte
		uses	bx, ds, si, es
		.enter
		call	SysLockBIOS	; grab this for the duration, to
					;  avoid any unpleasantness with
					;  TSRs and device drivers
	;
	; Initialize mapping state variables.
	; 
		movdw	ss:[lastBase], -1
		mov	ss:[lastSpace], 0
		mov	ss:[seenTuple], FALSE
	;
	; Find the location of the memory window.
	; 
		clr	al, bh		; adapter 0, window 0
		segmov	es, ss
		lea	di, ss:[window]
		mov	ss:[window].WD_bufferLength, size window - offset WD_table
		mov	ah, SS_INQUIRE_WINDOW
		int	SOCKET_SERV_INTERRUPT
		jc	done
	;
	; Make sure it can handle both attribute and common memory.
	; 
		mov	ax, SRC_INADEQUATE_WINDOW
		test	bl, mask GWC_COMMON or mask GWC_ATTRIBUTE
		stc
		jz	done		; neither
		jpo	done		; only one
	;
	; Compute the segment address for easy access.
	; 
		mov	ax, ss:[window].WD_table.WDT_mem.MWT_firstByte
		mov	ah, al
		clr	al
		mov	ss:[winSeg], ax
	;
	; Now bind the window to the socket.
	; 
		mov	ax, SS_SET_WINDOW shl 8
		clr	bx		; socket 0, window 0
		mov	cx, ss:[window].WD_table.WDT_mem.MWT_maxSize
		mov	dh, mask WS_ENABLE
		mov	dl, ss:[window].WD_table.WDT_mem.MWT_slowest
		mov	di, ss:[window].WD_table.WDT_mem.MWT_firstByte
		int	SOCKET_SERV_INTERRUPT
		jc	done
	;
	; Enumerate each CIS in turn. Initialize the longlink to offset 0
	; within common space. PS_ENABLED will be cleared if there is no
	; link.
	; 
		movdw	ss:[longLinkBase], 0
		mov	ss:[longLinkSpace], mask PS_ENABLED
		clrdw	cxdx
		mov	bl, mask PS_ATTRIBUTE
		mov	bh, TRUE		; CIS is primary
cisLoop:
		call	PCMEnumCISLow
		jc	done
		movdw	cxdx, ss:[longLinkBase]
		clr	bx		; no default link for any but primary
					;  bh <- CIS is secondary
		xchg	bl, ss:[longLinkSpace]
		test	bl, mask PS_ENABLED
		jnz	cisLoop
done:
		call	SysUnlockBIOS
		.leave
		ret
PCMEnumCIS		endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PCMEnumCISLow
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Enumerate a single CIS

CALLED BY:	(INTERNAL) PCMEnumCIS
PASS:		ss:bp	= frame inherited from PCMEnumCIS
		cxdx	= offset of CIS
		bl	= PageState indicating space in which CIS resides
		bh	= non-zero if CIS is primary, so first tuple must
			  be CISTPL_DEVICE
			= zero if CIS is secondary, so first tuple must be
			  CISTPL_LINKTARGET
RETURN:		carry set on error or if callback returned carry set
			ah	= SocketReturnCode (SRC_SUCCESS if callback
				  said to stop)
DESTROYED:	ax, bx, cx, dx, si, es
SIDE EFFECTS:	ss:[longLinkBase], ss:[longLinkSpace] may be altered

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/18/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
PCMEnumCISLow	proc	near
		.enter	inherit PCMEnumCIS
	;
	; Ensure initial tuple is valid.
	; 
		call	PCMECReadByte
		jc	toDone

		mov	ah, CISTPL_DEVICE
		tst	bh
		jnz	checkFirst
		mov	ah, CISTPL_LINKTARGET
checkFirst:
		cmp	al, ah
		je	initialCodeOK
		clr	ah
		tst	bh		; secondary?
		jz	ensureSeenSomething ; yes -- not an error, just stop
					;  enumerating

		cmp	al, CISTPL_END	; end tuple, so CIS should be in
					;  common?
		je	toDone		; yes -- leave seenTuple false, though
					;  so we take the lack of a
					;  secondary CIS as an error
noCisErr:
		mov	ah, SRC_NO_CIS	; no -- return error saying there's
		stc			;  no CIS
toDone:
		jmp	done

ensureSeenSomething:
		tst	ss:[seenTuple]
		jnz	toDone
		jmp	noCisErr

initialCodeOK:
		cmp	al, CISTPL_LINKTARGET
		jne	enumLoop
	;
	; Must do extra stuff to validate the LINKTARGET tuple:
	; 	- ensure link field >= 3
	; 	- ensure first 3 bytes are 'C', 'I', and 'S'
	; 
		pushdw	cxdx
		incdw	cxdx
		call	PCMECReadByte
		jc	linkFailed
		cmp	al, 3
		jb	linkFailed

		mov	bh, size cisString
		segmov	es, cs
		mov	di, offset cisString
checkCisStringLoop:
		incdw	cxdx			; fetch next byte
		call	PCMECReadByte
		jc	linkFailed

		scasb				; as expected?
		jne	linkFailed		; no

		dec	bh
		jnz	checkCisStringLoop
		popdw	cxdx
		jmp	enumLoop
linkFailed:
		popdw	cxdx
		clc
		jmp	toDone
cisString	char	"CIS"

	;--------------------
	; al	= CISTuple
	; cxdx	= position of same
	; bl	= PageState
enumLoop:
		mov	ss:[seenTuple], TRUE
	;
	; Stop processing if tuple is CISTPL_END
	;
		cmp	al, CISTPL_END
		je	done
		
		les	di, ss:[buffer]
	;
	; Fetch size byte.
	; 
		push	ax, di		; save for callback
		incdw	cxdx
		call	PCMECReadByte
		jc	popAndError

		mov	bh, al		; bh <- loop count
		clr	ah
		mov_tr	si, ax		; si <- count, for callback
		tst	bh
		jz	prepareForCallback	;Branch if no data
	;
	; Now fetch that many bytes o' data into the passed buffer.
	;

;
;	WARNING: We blindly read in any CIS information that we find.
;		The PCMCIA PC Card Standard warns on page 5-12 that we
;		should only read tuples that we know are safe - vendor
;		specific tuples should be skipped over, as they may have
;		active I/O registers mapped into those areas.
;

PrintMessage <Should skip over unknown CIS tuples here>
dataLoop:
		incdw	cxdx
		call	PCMECReadByte
		jc	popAndError
		stosb
		dec	bh
		jnz	dataLoop

prepareForCallback:
	;
	; Set up registers for calling the callback.
	; 
		pop	ax, di		; ax <- tuple code, di <- buffer base
		pushdw	cxdx		; save CIS position for loop

		mov	cx, si		; cx <- data size
	;
	; Filter out tuples having to do with chaining CIS's
	; 
		CheckHack <CISTPL_LONGLINK_C eq CISTPL_LONGLINK_A+1>
		CheckHack <CISTPL_LINKTARGET eq CISTPL_LONGLINK_C+1>
		CheckHack <CISTPL_NO_LINK eq CISTPL_LINKTARGET+1>

		cmp	al, CISTPL_NO_LINK
		ja	callCallback
		cmp	al, CISTPL_LONGLINK_A
		jae	copeWithLink

callCallback:
		push	si		; save data size for detecting end
		call	ss:[callback]
		pop	si

callbackDone:
		popdw	cxdx		; cxdx <- position of last byte read

		mov	ah, SRC_SUCCESS	; assume stopping
		jc	done
		cmp	si, CISTPL_END	; link field ff?
		je	done		; => done with CIS

		incdw	cxdx		; advance to next tuple
		call	PCMECReadByte	; al <- first byte of next
		jnc	enumLoop
done:
		.leave
		ret

popAndError:
		pop	di, di		; clear 2 words from stack
					;  without biffing carry
		jmp	done

copeWithLink:
	;
	; Deal with special tuples that only have meaning to us:
	; 	- CISTPL_NO_LINK: clear the longLinkSpace variable in
	;			  PCMEnumCIS so it knows there's not another CIS
	;
	;	- CISTPL_LONGLINK_C: copy bytes from buffer to longLinkBase and
	;			     set PS_ENABLED bit in longLinkSpace
	;
	;	- CISTPL_LONGLINK_A: copy bytes from buffer to longLinkBase and
	;			     set PS_ENABLED and PS_ATTRIBUTE bits in
	;			     longLinkSpace
	;
	;	- CISTPL_LINKTARGET: ignore it.
	;
		cmp	al, CISTPL_NO_LINK
		jne	checkLongLink

		mov	ss:[longLinkSpace], 0	; clear default link
		jmp	callbackDone		; (carry cleared by == compare)

checkLongLink:
		mov	bh, mask PS_ENABLED 	; assume link to common space
		cmp	al, CISTPL_LONGLINK_C
		je	haveLink
		cmp	al, CISTPL_LONGLINK_A
		jne	linkDone		; must be link target, which
						;  we just ignore
		ornf	bh, mask PS_ATTRIBUTE
haveLink:
		mov	ss:[longLinkSpace], bh

		clrdw	dxax			; assume fewer than 4 bytes...

	; (yeah, this is gross, but it handles all the cases I can think of,
	; and it doesn't trash SI)
		cmp	si, 0
		je	haveBase
		mov	al, es:[di+0]
		cmp	si, 1
		je	haveBase
		mov	ah, es:[di+1]
		cmp	si, 2
		je	haveBase
		mov	dl, es:[di+2]
		cmp	si, 3
		je	haveBase
		mov	dh, es:[di+3]
haveBase:
		movdw	ss:[longLinkBase], dxax
linkDone:
		clc				; don't stop processing
		jmp	callbackDone
PCMEnumCISLow	endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PCMECReadByte
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Read a byte from the CIS

CALLED BY:	(INTERNAL)
PASS:		ss:bp	= frame inherited from PCMEnumCIS
		bl	= PageState indicating from which address space to
			  fetch the byte
		cxdx	= offset from which to read it
RETURN:		al	= byte read
DESTROYED:	nothing
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/18/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
PCMECReadByte	proc	near
		uses	bx, cx, dx, si, di, ds
		.enter	inherit	PCMEnumCIS
	;
	; Multiply the address by 2 if in attribute space, where we get only
	; the even bytes.
	; 
		test	bl, mask PS_ATTRIBUTE
		jz	haveBase
		shldw	cxdx
haveBase:
	;
	; Save desired position for computing offset within window.
	; 
		mov	si, dx		; just need low word as we couldn't
					;  cope with a difference > 64K anyway
	;
	; Round down to nearest 4K, at least, or to window size, if required
	; 
		mov	ax, 4096
		test	ss:[window].WD_table.WDT_mem.MWT_capabilities,
				mask MWC_BASE_ALIGN
		jz	computeWinBase
		mov	ax, ss:[window].WD_table.WDT_mem.MWT_maxSize
		ror	ax		; shift it left 12 bits to
		ror	ax		;  multiply by 4K
		ror	ax
		ror	ax
computeWinBase:
		dec	ax
		not	ax
		andnf	dx, ax		; cxdx <- base for mapping
	;
	; If previous mapping was in a different space, must always map
	; 
		mov	bh, bl
		xor	bh, ss:[lastSpace]
		jnz	newMap
	;
	; Else only remap if previous base different from this one.
	; 
		cmpdw	ss:[lastBase], cxdx
		je	mapped
newMap:
	;
	; Remember new mapping for next time.
	; 
		movdw	ss:[lastBase], cxdx
		mov	ss:[lastSpace], bl
	;
	; Convert map base to 4K units (i.e. shrdw cxdx, 12)
	; 
		call	PCMConvertToPage
	;
	; Map the page in write-protected, just to be safe, since we'll
	; be reading only...
	; 
		mov	dl, bl		; dl <- PageState
		ornf	dl, mask PS_ENABLED
		test	ss:[window].WD_table.WDT_mem.MWT_capabilities,
			mask MWC_WRITE_PROTECT
		jz	mapIt
		ornf	dl, mask PS_WP
mapIt:
		clr	bx		; window 0, page 0
		mov	ax, SS_SET_PAGE shl 8
		int	SOCKET_SERV_INTERRUPT
		jc	done
mapped:
	;
	; Figure the offset within the window from which to fetch the
	; byte, and then do so.
	; 
		sub	si, ss:[lastBase].low
		mov	ds, ss:[winSeg]
		lodsb
;		clc
;check power - brianc 8/12/93
		call	PCMCheckCardPower	; returns carry
done:
		.leave
		ret
PCMECReadByte	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PCMCheckCardPower
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Check power to card

CALLED BY:	(INTERNAL)
PASS:		nothing
RETURN:		carry set on error
		carry clear if ok
DESTROYED:	none
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	brianc	8/12/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
PCMCheckCardPower	proc	near
		uses	ax, bx, cx, dx, di
		.enter
	;
	; Use GET_SOCKET to check power
	; 
		clr	bl		; socket is zero
		mov	ax, SS_GET_SOCKET shl 8		; adapter is zero
		int	SOCKET_SERV_INTERRUPT
					; returns: ah, bh, ch, cl, dh, dl, di
		jc	done
		cmp	cx, 0		; power off?
		stc			; assume so
		je	done		; yes
		clc			; else, okay
done:
		.leave
		ret
PCMCheckCardPower	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PCMConvertToPage
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Convert a 32-bit offset into a 4K-page number

CALLED BY:	(EXTERNAL)
PASS:		cxdx	= offset
RETURN:		di	= page number
		ax	= offset w/in page
DESTROYED:	ax
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	6/ 2/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
PCMConvertToPage proc	near
		uses	cx, dx
		.enter
		mov_tr	ax, cx
		mov	cl, 4
		rol	dx, cl		; high 4 to low 4
		andnf	dx, 0xf		; clear all but low 4
		shl	ax, cl		; make room for them in ax
		ornf	ax, dx		; and merge...
		mov_tr	di, ax		; di <- page
		.leave
		ret
PCMConvertToPage endp

Init	ends
