COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	Copyright (c) GeoWorks 1993 -- All Rights Reserved

PROJECT:	
MODULE:		
FILE:		nopowerDrive.asm

AUTHOR:		Adam de Boor, May 12, 1993

ROUTINES:
	Name			Description
	----			-----------

	
REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Tony	6/25/93		Initial revision


DESCRIPTION:
	This code handles notification that drive related events have
	happened.  The floppy drive A is used to simulate a removable PCMCIA
	drive.

	$Id: nppcmDrive.asm,v 1.1.1.1 93/09/27 13:26:55 adam Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

include Internal/im.def
include thread.def
include Internal/fsd.def
include system.def

;------------------------------------------------------------------------------
;		Constants
;------------------------------------------------------------------------------

CARD_INSERT_DELAY	equ	6		;1/10th of a second

CARD_EJECT_SLEEP_TIME	equ	5*60	; 5 seconds between checks

DRIVE_UNREGISTER_RETRY_COUNT	equ	6	; if all things not gone after

;------------------------------------------------------------------------------
;		Classes
;------------------------------------------------------------------------------

; A thread with this class is created when a new card is inserted

NoPowerCardInsertClass	class	ProcessClass

MSG_NPCI_ADD_DRIVE				message
;
;	Desc:	Check to see if the newly-inserted drive card has anything
;		we can use.
;	Pass:	nothing
;	Return:	nothing
;	Destroy:	anything
;

NoPowerCardInsertClass	endc

;-------------------------------------------

; A thread with this class is created when a card is ejected

NoPowerCardEjectClass class	ProcessClass

MSG_NPCE_CARD_IS_BACK				message
;
;	Desc:	Sent when error box has been acknowledged by the user and,
;		presumably, the card is back in the slot. We wait until we
;		receive this to notify the world of the drive's absence, to
;		avoid generating a bunch of SysNotify messages on top of our
;		own box as all the things try to close their files at once.
;	Pass:	nothing
;	Return:	nothing
;	Destroy:	anything
;

MSG_NPCE_TIMER_EXPIRED				message
;
;	Desc:	Check to see if the card is no longer in use and tell the user
;		if it's safe to remove it.
;
;	Pass:	nothing
;	Return:	nothing
;	Destroy:	anything

NoPowerCardEjectClass endc

;------------------------------------------------------------------------------
;		Variables
;------------------------------------------------------------------------------

idata	segment

NoPowerCardInsertClass	mask CLASSF_NEVER_SAVED
NoPowerCardEjectClass	mask CLASSF_NEVER_SAVED

insertEjectSem	Semaphore

driveAPrivateData	word	0
diskHandle		word	0

diskType DriveExtendedStatus	mask DES_FORMATTABLE or mask DES_LOCAL_ONLY or \
			(mask DS_PRESENT or mask DS_MEDIA_REMOVABLE or \
			 (DRIVE_5_25 shl offset DS_TYPE)) shl \
			 offset DES_EXTERNAL

idata	ends

;---

udata	segment
udata	ends

;------------------------------------------------------------------------------
;		Code
;------------------------------------------------------------------------------

Movable		segment	resource

COMMENT @----------------------------------------------------------------------

FUNCTION:	NoPowerDriveInit

DESCRIPTION:	Initialize support for the drive

CALLED BY:	INTERNAL

PASS:
	ds, es - idata

RETURN:
	none

DESTROYED:
	ax, bx, cx, dx, si, di, bp

REGISTER/STACK USAGE:

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/CAVEATS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Tony	6/25/93		Initial version

------------------------------------------------------------------------------@
NoPowerDriveInit	proc	far

	call	NoPowerNukeDriveA

	ret

NoPowerDriveInit	endp

COMMENT @----------------------------------------------------------------------

FUNCTION:	NoPowerNukeDriveA

DESCRIPTION:	Nuke drive A

CALLED BY:	INTERNAL

PASS:
	ds, es - idata

RETURN:
	none

DESTROYED:
	ax, bx, cx, dx, si, di, bp

REGISTER/STACK USAGE:

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/CAVEATS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Tony	6/25/93		Initial version

------------------------------------------------------------------------------@
NoPowerNukeDriveA	proc	near	uses ds
	.enter

	; Hack here -- Save the private data offset for later

	call	FSDLockInfoExcl
	mov	ds, ax
	mov	si, ds:[FIH_driveList]
findLoop:
	tst	si
	jz	done
	cmp	ds:[si].DSE_number, 0
	jz	driveFound
	mov	si, ds:[si].DSE_next
	jmp	findLoop
driveFound:

	clr	ax
	xchg	ax, ds:[si].DSE_private
	call	FSDUnlockInfoExcl

	mov	bx, idata
	mov	ds, bx
	mov	ds:[driveAPrivateData], ax
	mov	ds:[diskHandle], 0

	; Delete the A drive, which does not actually exist (until the user
	; says it does)
 
	mov	al, 0
	call	FSDDeleteDrive

done:
	.leave
	ret

NoPowerNukeDriveA	endp

Movable		ends

Resident	segment	resource

COMMENT @----------------------------------------------------------------------

FUNCTION:	NoPowerDriveExit

DESCRIPTION:	...

CALLED BY:	INTERNAL

PASS:
	ds, es - idata

RETURN:
	none

DESTROYED:
	ax, bx, cx, dx, si, di, bp

REGISTER/STACK USAGE:

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/CAVEATS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Tony	6/25/93		Initial version

------------------------------------------------------------------------------@
NoPowerDriveExit	proc	near
	ret

NoPowerDriveExit	endp

Resident	ends

;===========================================================

Movable	segment	resource

COMMENT @----------------------------------------------------------------------

FUNCTION:	NoPowerDriveNukeThread

DESCRIPTION:	Force-queue a message to the current thread to destroy it.

CALLED BY:	INTERNAL

PASS:
	none

RETURN:
	none

DESTROYED:
	ax, bx, cx, dx, si, di, bp, ds, es

PASS:

RETURN:

DESTROYED:

REGISTER/STACK USAGE:

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/CAVEATS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Tony	6/25/93		Initial version

------------------------------------------------------------------------------@
NoPowerDriveNukeThread	proc	near

	; Fetch the handle of the current thread.
 
	mov	ax, TGIT_THREAD_HANDLE
	clr	bx
	call	ThreadGetInfo

	; Force-queue a detach to ourselves. This will nuke us eventually...
 
	mov	ax, MSG_META_DETACH
	clr	cx, dx, bp
	mov	di, mask MF_FORCE_QUEUE
	call	ObjMessage

	ret

NoPowerDriveNukeThread	endp

Movable ends

Resident	segment	resource

COMMENT @----------------------------------------------------------------------

FUNCTION:	NoPowerDriveInserted

DESCRIPTION:	Handle the drive being "inserted"

CALLED BY:	Keyboard simulation code (at interrupt level)

PASS:
	none

RETURN:
	none

DESTROYED:
	none

REGISTER/STACK USAGE:

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/CAVEATS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Tony	6/25/93		Initial version

------------------------------------------------------------------------------@
NoPowerDriveInserted	proc	near
	push	dx
	mov	dx, offset NoPowerCardInsertClass
	call	NoPowerDriveCreateEventThread
	pop	dx
	ret

NoPowerDriveInserted	endp

COMMENT @----------------------------------------------------------------------

FUNCTION:	NoPowerDriveEjected

DESCRIPTION:	Handle the drive being "inserted"

CALLED BY:	Keyboard simulation code (at interrupt level)

PASS:
	none

RETURN:
	none

DESTROYED:
	none

REGISTER/STACK USAGE:

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/CAVEATS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Tony	6/25/93		Initial version

------------------------------------------------------------------------------@
NoPowerDriveEjected	proc	near
	push	dx
	mov	dx, offset NoPowerCardEjectClass
	call	NoPowerDriveCreateEventThread
	pop	dx
	ret

NoPowerDriveEjected	endp

COMMENT @----------------------------------------------------------------------

FUNCTION:	NoPowerDriveCreateEventThread


DESCRIPTION:	Use the input manager to create an event-driven thread for us

CALLED BY:	INTERNAL

PASS:
	dx - offset within dgroup of subclass of GenProcessClass to use

RETURN:
	none

DESTROYED:
	none

REGISTER/STACK USAGE:

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/CAVEATS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Tony	6/25/93		Initial version

------------------------------------------------------------------------------@
NoPowerDriveCreateEventThread	proc	near uses ax, bx, cx, dx, si, di, bp, ds
	.enter

	; Create an event thread in the input manager to do the work,
	; since we cannot do any substantial work in interrupt code.

	call	ImInfoInputProcess
	mov	ax, MSG_PROCESS_CREATE_EVENT_THREAD
	mov	bp, 1024		;bp <- stack size
	mov	cx, dgroup
	tst	bx
	jz	noIM
	mov	di, mask MF_FORCE_QUEUE
	call	ObjMessage
done:
	.leave
	ret

noIM:
	;
	; Must be happening at the start, so use ObjCallClassNoLock instead.
	; 
	mov	bx, handle geos
	call	GeodeGetDGroupDS
	movdw	esdi, cxdx
	call	ObjCallClassNoLock
	jmp	done

NoPowerDriveCreateEventThread	endp

Resident	ends

Movable	segment	resource

COMMENT @----------------------------------------------------------------------

MESSAGE:	NoPowerCardInsertAttach -- MSG_META_ATTACH
						for NoPowerCardInsertClass

DESCRIPTION:	Handle attach for our drive register mechanism

PASS:
	*ds:si - instance data
	es - segment of NoPowerCardInsertClass

	ax - The message

RETURN:

DESTROYED:
	bx, si, di, ds, es (message handler)

REGISTER/STACK USAGE:

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/CAVEATS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Tony	6/25/93		Initial version

------------------------------------------------------------------------------@
NoPowerCardInsertAttach	method dynamic	NoPowerCardInsertClass, MSG_META_ATTACH

	PSem	es, insertEjectSem, TRASH_AX_BX

	; Send ourselves a message, delayed via a timer.  The delay is
	; to allow the PCMCIA card to fully power itself up.

	mov	dx, MSG_NPCI_ADD_DRIVE
	mov	ax, TGIT_THREAD_HANDLE
	clr	bx
	call	ThreadGetInfo			;bx <- current thread

	mov	al, TIMER_EVENT_ONE_SHOT
	mov	cx, CARD_INSERT_DELAY
	call	TimerStart

	ret

NoPowerCardInsertAttach	endm

COMMENT @----------------------------------------------------------------------

MESSAGE:	NoPowerCardInsertAddDrive -- MSG_NPCI_ADD_DRIVE
						for NoPowerCardInsertClass

DESCRIPTION:	Add the drive

PASS:
	*ds:si - instance data
	es - segment of NoPowerCardInsertClass

	ax - The message

RETURN:

DESTROYED:
	bx, si, di, ds, es (message handler)

REGISTER/STACK USAGE:

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/CAVEATS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Tony	6/25/93		Initial version

------------------------------------------------------------------------------@
idata	segment
driveName	char	"A", 0
idata	ends

NoPowerCardInsertAddDrive	method dynamic	NoPowerCardInsertClass,
						MSG_NPCI_ADD_DRIVE

	; get the offset of the primary FSD

	call	FSDLockInfoShared
	mov	ds, ax
	mov	dx, ds:[FIH_primaryFSD]
	call	FSDUnlockInfoShared

	; Create the drive descriptor first.

	segmov	ds, es
	mov	bx, ds:[driveAPrivateData]
	clr	al
	mov	ah, MEDIA_CUSTOM
	mov	cx, ds:[diskType]
	mov	si, offset driveName
	call	FSDInitDrive

	; Try and register the disk. Do it silently, as the user usually won't
	; be seeing prompts to put the card back in, except from us.

	clr	al
	call	DiskRegisterDiskSilently
	jc	done

	mov	ds:[diskHandle], bx

	; Got it. Try and change to the top-level directory we've defined
	; as containing our stuff.

	segmov	ds, cs
	mov	dx, offset topDirSpec
	call	FileSetCurrentPath
	jc	done

	; Add <drive>:\GEOWORKS as a StandardPath

	mov	cx, size PathName	; cx <- buffer size
	sub	sp, cx
	mov	di, sp
	segmov	es, ss			; es:di <- buffer
	mov	si, dx			; ds:si <- tail
	mov	dx, TRUE		; want drive name
	call	FileConstructFullPath

	segmov	ds, ss
	mov	dx, sp			;ds:dx <- path to add
	mov	ax, SP_TOP		;ax <- StandardPath
	clr	bx			;bx <- FileAddStandardPathFlags
	call	FileAddStandardPathDirectory
	add	sp, size PathName
done:

	call	NoPowerDriveNukeThread

	ret

NoPowerCardInsertAddDrive	endm

topDirSpec	char	'\\GEOWORKS', 0

COMMENT @----------------------------------------------------------------------

MESSAGE:	NoPowerCardInsertDetach -- MSG_META_DETACH
					for NoPowerCardInsertClass

DESCRIPTION:	Go away

PASS:
	*ds:si - instance data
	es - segment of NoPowerCardInsertClass

	ax - The message

RETURN:

DESTROYED:
	bx, si, di, ds, es (message handler)

REGISTER/STACK USAGE:

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/CAVEATS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Tony	6/25/93		Initial version

------------------------------------------------------------------------------@
NoPowerCardInsertDetach	method dynamic	NoPowerCardInsertClass, MSG_META_DETACH
	VSem	es, insertEjectSem
	mov	di, offset NoPowerCardInsertClass
	GOTO	ObjCallSuperNoLock

NoPowerCardInsertDetach	endm

COMMENT @----------------------------------------------------------------------

MESSAGE:	NoPowerCardEjectAttach -- MSG_META_ATTACH
					for NoPowerCardEjectClass

DESCRIPTION:	Handle attach for our card removal mechanism

PASS:
	*ds:si - instance data
	es - segment of NoPowerCardEjectClass

	ax - The message

RETURN:

DESTROYED:
	bx, si, di, ds, es (message handler)

REGISTER/STACK USAGE:

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/CAVEATS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Tony	6/25/93		Initial version

------------------------------------------------------------------------------@
NoPowerCardEjectAttach	method dynamic	NoPowerCardEjectClass, MSG_META_ATTACH

	segmov	ds, es
	PSem	ds, insertEjectSem, TRASH_AX_BX

	; See if the disk that was there is still in-use
 
	call	NoPowerDriveCheckInUse
	jc	cardIsInUse

	call	NoPowerDriveSuicide
	ret

cardIsInUse:

	; It was. Whine at the user. Can't use existing code, sadly, as it
	; has no way of passing back a message when the box comes down...
 
	push	bx
	mov	ax, TGIT_THREAD_HANDLE
	clr	bx
	call	ThreadGetInfo
	mov_tr	cx, ax
	pop	bx

	mov	di, MSG_NPCE_CARD_IS_BACK
	mov	ax, mask CDBF_SYSTEM_MODAL or \
		    (CDT_ERROR shl offset CDBF_DIALOG_TYPE) or \
		    (GIT_NOTIFICATION shl offset CDBF_INTERACTION_TYPE)
	call	NoPowerDriveDisplayBXSI
	ret

NoPowerCardEjectAttach	endm

COMMENT @----------------------------------------------------------------------

FUNCTION:	NoPowerDriveDisplayBXSI

DESCRIPTION:	Put up the message passed in ^lbx:si

CALLED BY:	NoPowerCardEjectAttach

PASS:
	^lbx:si - string to display
	ax - CustomDialogBoxFlags
	^lcx:dx - finishOD for the dialog (0 if no response required)
	di - message to send	

RETURN:
	none

DESTROYED:
	ax, bx, cx, dx, si, di, bp

REGISTER/STACK USAGE:

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/CAVEATS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Tony	6/25/93		Initial version

------------------------------------------------------------------------------@
NoPowerDriveDisplayBXSI	proc	near
	sub	sp, size GenAppDoDialogParams
	mov	bp, sp
	mov	ss:[bp].GADDP_dialog.SDP_customFlags, ax

	; Deref the error string.
 
	call	MemLock
	mov	ds, ax
	mov	si, ds:[si]
	movdw	ss:[bp].GADDP_dialog.SDP_customString, axsi
	push	bx

	; Zero-init things not used.
 
	clr	bx
	clrdw	ss:[bp].GADDP_dialog.SDP_stringArg1, bx
	clrdw	ss:[bp].GADDP_dialog.SDP_stringArg2, bx
	clrdw	ss:[bp].GADDP_dialog.SDP_customTriggers, bx
	clrdw	ss:[bp].GADDP_dialog.SDP_helpContext, bx

	; Send us a message when the box comes down.
 
	movdw	ss:[bp].GADDP_finishOD, cxdx
	mov	ss:[bp].GADDP_message, di

	; Locate the UI's application object.
 
	mov	ax, SGIT_UI_PROCESS
	call	SysGetInfo
	mov_tr	bx, ax
	call	GeodeGetAppObject

	; Send it the message to put the box up, thanks.
 
	mov	ax, MSG_GEN_APPLICATION_DO_STANDARD_DIALOG
	mov	dx, size GenAppDoDialogParams
	mov	di, mask MF_CALL or mask MF_STACK
	call	ObjMessage

	pop	bx		; bx <- error message handle
	add	sp, size GenAppDoDialogParams
	call	MemUnlock

	ret

NoPowerDriveDisplayBXSI	endp

COMMENT @----------------------------------------------------------------------

FUNCTION:	NoPowerDriveCheckInUse

DESCRIPTION:	See if the disk registered for our drive is currently in-use

CALLED BY:	NoPowerCardEjectAttach

PASS:
	ds, es - dgroup

RETURN:
	carry set if is in-use:
		^lbx:si	 = string to use in error box
	carry clear if all-clear.
		^lbx:si	 = string to use if eject was denied before

DESTROYED:
	ax

REGISTER/STACK USAGE:

PSEUDO CODE/STRATEGY:
		check if any disk for drive in-use
		if not:
		    if no complaints from kids:
			unload them all
			free block holding them
			remove standard path
			delete drive
			unregister as FSD
			return CF=0 and BX=0 (ignore any message from kids)
		    else
			return CF=1 and last message returned by kid
		    fi
		else
		    decrement ffsDiskCheckCount
		    if now 0, reboot system
		    return CF=1 and bx:si set to our message
		fi

KNOWN BUGS/SIDE EFFECTS/CAVEATS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Tony	6/25/93		Initial version

------------------------------------------------------------------------------@
NoPowerDriveCheckInUse	proc	near	uses	ax, cx, dx
	.enter

	; Check if any disk for the drive is in-use. Only on first time do
	; driver files count as uses for the disk.

	mov	bx, ds:[diskHandle]
	tst	bx
	jz	prepareForUnload
	call	NPDEDiskCheckInUse
	jnc	prepareForUnload
		
	mov	bx, handle CardStillInUseString
	mov	si, offset CardStillInUseString
done:
	.leave
	ret

prepareForUnload:

	; Neither we nor the card driver will be needed after this, so 
	; unload it, and force-queue a message to ourselves to go away.

 	mov	di, cs
	mov	si, offset NPDENotifyAnyway
	call	DiskForEach

	; Since the disk isn't in-use, nuke any standard path, nuke the drive,
	; then unregister ourselves as an FSD

	call	NPDEDeleteStdPathForAllDrives

	call	NoPowerNukeDriveA

	clr	bx			; no message, clears carry
	jmp	done

NoPowerDriveCheckInUse	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		NPDEDiskCheckInUse
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Determine if a disk handle is actively in-use, either by
		an open file or by a thread having a directory on the disk
		in its directory stack. Don't count the file handles open
		for any of our alternate drivers, however.

CALLED BY:	GLOBAL
PASS:		bx	= disk handle
RETURN:		carry set if the disk is in-use.
DESTROYED:	nothing

PSEUDO CODE/STRATEGY:
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/24/90		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
NPDEDiskCheckInUse proc	near
		uses	ax, cx, di, si, bx
		.enter
		mov	cx, bx
		mov	di, cs
		mov	si, offset DIU?_fileCallback
		clr	bx		; process whole list, please
		call	FileForEach
		jc	done
		
		mov	di, cs
		mov	si, offset DIU?_pathCallback
		call	FileForEachPath
done:
		.leave
		ret
NPDEDiskCheckInUse endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		DIU?_fileCallback
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Callback routine to determine if a file is open to a disk.

CALLED BY:	DiskCheckInUse via FileForEach
PASS:		bx	= file handle
		ds	= kdata
		cx	= disk handle being sought
RETURN:		carry set if file open on disk. This will stop traversal
		and cause a carry-set return from DiskCheckInUse
DESTROYED:	nothing

PSEUDO CODE/STRATEGY:
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	3/ 6/92		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
DIU?_fileCallback	proc far

	; See if file open to this disk.
 
	cmp	cx, ds:[bx].HF_disk
	stc
	jz	done
	clc
done:
	ret

DIU?_fileCallback endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		DIU?_pathCallback
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Callback routine to determine if a path in a thread's
		directory stack is on a particular disk.

CALLED BY:	DiskCheckInUse via FileForEach
PASS:		bx	= path handle
		di	= disk handle for the path
		cx	= disk handle being sought
RETURN:		carry set if path is on the disk. This will stop traversal
		and cause a carry-set return from DiskCheckInUse
DESTROYED:	nothing

PSEUDO CODE/STRATEGY:
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	3/ 6/92		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
DIU?_pathCallback	proc far
	cmp	cx, di
	stc
	jz	done
	clc
done:
	ret
DIU?_pathCallback	endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		NPDEDeleteStdPathForAllDrives
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Nuke any std path we added for any of our possible drives.

CALLED BY:	(INTERNAL) NPDEConfirmRemoval, NPDECardReinserted
PASS:		ds	= dgroup
RETURN:		nothing
DESTROYED:	ax, bx, cx, dx, si, di
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	6/ 3/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
NPDEDeleteStdPathForAllDrives proc	near

	mov	al, 0
	call	NPDEDeleteStdPath

	ret
NPDEDeleteStdPathForAllDrives endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		NPDEDeleteStdPath
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Remove any std path we added.

CALLED BY:	(INTERNAL) NPDEConfirmRemoval, NPDECardReinserted
PASS:		ds	= dgroup
		al	= drive number
RETURN:		nothing
DESTROYED:	ax, bx, cx, dx, si, di
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/14/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
NPDEDeleteStdPath proc	near
		uses	ds, es
		.enter
	;
	; Make a buffer on le stack.
	; 
		mov	cx, size PathName	; cx <- buffer size
		sub	sp, cx
	;
	; Get the name of the drive.
	; 
		mov	di, sp
		segmov	es, ss			; es:di <- buffer
		call	DriveGetName
		jc	done
	;
	; Append a colon.
	; 
		mov	al, ':'
		stosb
	;
	; Copy in the topDirSpec.
	; 
		segmov	ds, cs
		mov	si, offset topDirSpec	; ds:si <- tail
		mov	cx, size topDirSpec
		rep	movsb
	;
	; Now remove that path from SP_TOP, if it's there.
	; 
		segmov	ds, ss
		mov	dx, sp			;ds:dx <- path to remove
		mov	ax, SP_TOP		;ax <- StandardPath
		call	FileDeleteStandardPathDirectory
done:
		add	sp, size PathName
		.leave
		ret
NPDEDeleteStdPath endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		NoPowerDriveCardIsBack
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Now that the card is supposedly back in the slot, perform
		the initial call to, tell
		everyone to stop using it.

CALLED BY:	MSG_CDU_CARD_IS_BACK
PASS:		es	= dgroup
RETURN:		nothing
DESTROYED:	anything
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/10/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
NoPowerDriveCardIsBack method NoPowerCardEjectClass, MSG_NPCE_CARD_IS_BACK
		.enter

	;
	; Tell the driver the card is back and see what it wants us to do.
	; 
	call	NPDECardReinserted
	jc	suicide			; nothing it can do, so just die,
					; now the card is back in.
	;
	; Perform initial check. This will start the timer going.
	; 
		clr	bx
		mov	ax, TGIT_THREAD_HANDLE
		call	ThreadGetInfo
		mov	ax, MSG_NPCE_TIMER_EXPIRED
		clr	di
		call	ObjMessage
done:
		.leave
		ret
suicide:
		call	NoPowerNukeDriveA
		call	NoPowerDriveSuicide
		jmp	done
NoPowerDriveCardIsBack endm



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		NPDECardReinserted
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Now that the disk is supposedly back in the slot, tell
		everyone to stop using it.

CALLED BY:	NoPowerDriveCardIsBack
PASS:		ds, es	= dgroup
RETURN:		carry set if there's nothing to be done
DESTROYED:	di, ds, es, ax
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		foreach alt driver
		    if not marked happy:
			call CARD_REINSERTED
			remember if CF=1
		    fi
		done
		if any child returned CF=1:
		    reconnect any happy drivers
		    clear happy bits
		    return CF=1
		fi
		find any in-use disk & generate notification, if found one
		return CF=0
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/10/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
NPDECardReinserted proc far	uses	bx, cx, dx, si
	.enter

	call	NPDEDeleteStdPathForAllDrives

	;
	; Send out notification for all disks we handle, regardless of whether
	; they are in-use.
	; 
	mov	di, cs
	mov	si, offset NPDENotifyAnyway
	call	DiskForEach

	.leave
	ret
NPDECardReinserted endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		NPDENotifyAnyway
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	If the disk isn't stale, and is for our drive, send out
		notification that it's going away.

CALLED BY:	NPDEConfirmRemoval via DiskForEach
PASS:		es:bx	= DiskDesc
RETURN:		carry set to stop processing
DESTROYED:	di, si
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/20/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
NPDENotifyAnyway proc	far
		.enter
		mov	si, es:[bx].DD_drive
		tst	si
		jz	done			; => stale disk
		cmp	es:[si].DSE_number, 0	; ours ?
		jne	done
		push	ax, bx, cx, dx, bp
		call	NPDENotify
		pop	ax, bx, cx, dx, bp
done:
		clc
		.leave
		ret
NPDENotifyAnyway endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		NPDENotify
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Notify everyone to stop using the disk.

CALLED BY:	(INTERNAL) NPDECardReinserted
PASS:		bx	= handle of disk being removed
RETURN:		nothing
DESTROYED:	ax, bx, cx, dx, si, di
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/20/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
NPDENotify	proc	near
		.enter
	;
	; Send out a MSG_META_REMOVING_DISK to tell everyone to stop using
	; the thing.
	; 
		mov	cx, bx
		clr	dx			; no data block.
		mov	bx, MANUFACTURER_ID_GEOWORKS
		mov	si, GCNSLT_REMOVABLE_DISK
		mov	ax, MSG_META_REMOVING_DISK
		mov	di, mask GCNLSF_FORCE_QUEUE
		push	cx
		call	GCNListRecordAndSend
		pop	cx
	;
	; Send out a MSG_META_REMOVING_DISK to tell everyone to stop using
	; the thing.
	; 
		clr	dx			; no data block.
		mov	bx, MANUFACTURER_ID_GEOWORKS
		mov	si, GCNSLT_FILE_SYSTEM
		mov	ax, MSG_META_REMOVING_DISK
		mov	di, mask GCNLSF_FORCE_QUEUE
		push	cx
		call	GCNListRecordAndSend
		pop	cx
	;
	; To make life simpler for people, also broadcast the message to all
	; owners of files open to the disk.
	; 
		mov	di, cs
		mov	si, offset NPDECR_callback
		clr	bx			; process entire list
		call	FileForEach
		.leave
		ret
NPDENotify	endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		NPDECR_callback
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Callback function to notify the owners of all files open to
		the in-use disk that's being removed.

CALLED BY:	NPDECardReinserted via FileForEach
PASS:		bx	= handle of open file
		ds	= kdata
		cx	= handle of affected disk
RETURN:		carry set to stop
DESTROYED:	di, ax
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/13/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
NPDECR_callback	proc	far
		uses	bx
		.enter
		cmp	ds:[bx].HF_disk, cx	; on affected disk?
		jne	done			; no

		mov	bx, ds:[bx].HF_owner
		mov	ax, GGIT_ATTRIBUTES
		call	GeodeGetInfo
		test	ax, mask GA_PROCESS	; is owner a process?
		jz	done			; no
		
		cmp	bx, handle geos		; is owner the kernel?
		je	done			; yes -- not a real process
	;
	; Queue it a message, then.
	; 
		mov	ax, MSG_META_REMOVING_DISK
		mov	di, mask MF_FORCE_QUEUE
		call	ObjMessage
done:
		clc
		.leave
		ret
NPDECR_callback	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		NPDEDisplayBXSIAndWait
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Put up the message in ^lbx:si as an error and wait for the
		user to acknowledge it.

CALLED BY:	(INTERNAL) NPDERebootSystem, NPDEUnloadAlternates
PASS:		^lbx:si	= string to display as an error notification
RETURN:		nothing
DESTROYED:	ax, bx, si, cx, dx, di
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/13/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
NPDEDisplayBXSIAndWait proc	near
		uses	bp
		.enter
	;
	; Allocate a queue to which the message saying the box is down can be
	; sent.
	; 
		push	bx
		call	GeodeAllocQueue			;bx = new queue
	;
	; Put up the box telling the user we must reboot.
	; 
		mov	cx, bx				;cx = new queue
		pop	bx				;bxsi = message
		push	cx
		mov	ax, mask CDBF_SYSTEM_MODAL or \
			    (CDT_ERROR shl offset CDBF_DIALOG_TYPE) or \
			    (GIT_NOTIFICATION shl offset CDBF_INTERACTION_TYPE)
		clr	di
		call	NoPowerDriveDisplayBXSI
	;
	; Wait for the box to come down.
	; 
		pop	bx
		call	QueueGetMessage
	;
	; Nuke the queue and the message.
	; 
		call	GeodeFreeQueue
		mov_tr	bx, ax
		call	ObjFreeMessage
		.leave
		ret
NPDEDisplayBXSIAndWait endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		NoPowerDriveTimerExpired
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	See if the current card is still in-use. If not let, the user
		know it's safe to remove the card, else...

CALLED BY:	MSG_NPCE_TIMER_EXPIRED
PASS:		es 	= dgroup
RETURN:		nothing
DESTROYED:	anything
SIDE EFFECTS:	a message is put up if the driver says it's ok

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/10/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
NoPowerDriveTimerExpired method NoPowerCardEjectClass, MSG_NPCE_TIMER_EXPIRED
		.enter

		segmov	ds, es
		call	NoPowerDriveCheckInUse
		jc	tryAgain
		
		tst	bx		; any message returned?
		jnz	notifyUser

		mov	bx, handle CardMayBeRemovedString
		mov	si, offset CardMayBeRemovedString
notifyUser:
		mov	ax, mask CDBF_SYSTEM_MODAL or \
			    (CDT_NOTIFICATION shl offset CDBF_DIALOG_TYPE) or \
			    (GIT_NOTIFICATION shl offset CDBF_INTERACTION_TYPE)
		clr	cx, dx			; no finish OD
		clr	di
		call	NoPowerDriveDisplayBXSI
	;
	; We have no more reason to exist, so queue the message that will
	; destroy us.
	; 
		call	NoPowerDriveSuicide
done:
		.leave
		ret
tryAgain:
	;
	; Start up another timer to tell us to check again in a bit.
	; 
		mov	ax, TGIT_THREAD_HANDLE
		clr	bx
		call	ThreadGetInfo

		mov	al, TIMER_EVENT_ONE_SHOT
		mov	cx, CARD_EJECT_SLEEP_TIME
		mov	dx, MSG_NPCE_TIMER_EXPIRED
		call	TimerStart
		jmp	done
NoPowerDriveTimerExpired endm


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		NoPowerDriveSuicide
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Unload the current card driver and kill the current thread

CALLED BY:	(INTERNAL) NoPowerDriveTimerExpired, NoPowerCardEjectAttach
PASS:		es	= dgroup
RETURN:		nothing
DESTROYED:	bx
SIDE EFFECTS:	a MSG_META_DETACH is queued for the current thread

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/13/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
NoPowerDriveSuicide proc	near
		.enter
		call	NoPowerDriveNukeThread
		.leave
		ret
NoPowerDriveSuicide endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		NoPowerCardEjectDetach
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	

CALLED BY:	MSG_META_DETACH
PASS:		
RETURN:		
DESTROYED:	
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/21/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
NoPowerCardEjectDetach method dynamic NoPowerCardEjectClass, MSG_META_DETACH
		VSem	es, insertEjectSem
		mov	di, offset NoPowerCardEjectClass
		GOTO	ObjCallSuperNoLock
NoPowerCardEjectDetach		endm
Movable ends
