COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PROJECT:	PC GEOS
MODULE:		Com43 -- Communications program for MS-DOS entry and exit
FILE:		main.asm

AUTHOR:		Adam de Boor

ROUTINES:
	Name		Description
	----		-----------
	Main		Entry point
	Exit		Exit point
	SetInterrupt	Set an interrupt vector to one of our own routines
	ResetInterrupt	Restore an interrupt vector to what it used to be.

	
REVISION HISTORY:
	Name	Date	Description
	----	----	-----------
	adam	6/7	Initial Revision


DESCRIPTION:
	This program is used to communicate between a PC and another
	system -- in this case UNIX. It performs most simple terminal
	functions as well as file transfer. The file transfer is performed
	using a very simple protocol.

	A termcap entry is located in com43.ter.
		
	$Id: common.asm,v 1.8 94/01/22 14:46:08 cassie Exp $
	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
include com43.def

program		segment 


	; used to distinguish the two programs for slightly different
	; behavoir

pc_unix		db	?
err		db	?


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		CheckIsPC
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Checks to see if we are running on a base pc. If we are it
		sets the 'isBasePC' flag to 80h.

CALLED BY:	Main
PASS:		nothing
RETURN:		nothing
DESTROYED:	

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jcw	1/23/89		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	CheckIsPC
CheckIsPC	proc	near
		;
		; Determine whether we are on PC or AT to handle different
		; keyboards and baud rates.  On the AT the KBD_CONTROL
		; location can not have its high bit changed. So we try and
		; change it. If it changes, voila, its not an AT.
		;
		and	sysFlags, NOT MASK SF_PC; Init to be on an AT.
						;
		in	al,KBD_CONTROL      	; get special info
		mov	ah,al			; save info it ah
		or	al,080h			; set high bit 
		out	KBD_CONTROL,al		; 
		in	al,KBD_CONTROL      	; read back new special info
		xchg	al,ah			; save our changed? results
		out	KBD_CONTROL,al		; and return kbd to orig state
		test	ah,080h			; check if high bit changed
		jz	CIPC_isAT		; bra if not to AT init stuff
		or	sysFlags, MASK SF_PC	; flag as PC kbd
CIPC_isAT:					;
		ret				;
CheckIsPC	endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		SetInterrupt
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set an interrupt w/o consulting MS-DOS, saving the old value

CALLED BY:	ComInit, KbdInit

PASS:		ax = interrupt number to alter
		bx = place (4-byte) to store old value
		dx = new offset

RETURN:		Nothing

DESTROYED:	ax, dx


PSEUDO CODE/STRATEGY:
	Sets es to segment 0, copies old value out, places new offset and cs
	in vector.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

SetInterrupt	proc	near
		push	es		; preserve registers we'll use
		push	cx
		push	bp

		clr	cx		; Place segment 0 in es.
		mov	es, cx

		shl	ax,1		; Multiply vector # by 4 to get vector
		shl	ax,1		; address and place that in bp so we've
		mov	bp, ax		; got a pointer register to it

		dsi			; no interrupts while we're adjusting
					; the vectors, thank you.

		;
		; First see if we've already got the vector (for
		; re-initialization).
		; 
		mov	ax, cs
		cmp	es:2[bp], ax
		je	SIDoNothing	; Already ours -- do nothing

		xchg	es:2[bp], ax	; Replace segment value and get old.
		mov	2[bx], ax
		
		xchg	dx, es:[bp]	; Replace offset value and get old
		mov	[bx], dx
SIDoNothing:
		eni			; Can now allow interrupts

		pop	bp		; Restore trashed registers
		pop	cx
		pop	es
		ret
SetInterrupt	endp
		bugsym	SetInterrupt



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ResetInterrupt
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Reset an interrupt vector we modified

CALLED BY:	ComExit, KbdExit

PASS:		ax = vector number to reset
		bx = area (4-byte) from which to restore it

RETURN:		Nothing

DESTROYED:	ax

PSEUDO CODE/STRATEGY:
	Make es point to segment 0 and copy the old vector in w/o regard for
	what's there.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ResetInterrupt	proc	near
		push	es		; Preserve registers we'll use
		push	bp
		push	cx

		shl	ax,1		; Form address for vector and place in
		shl	ax,1		; bp so we can indirect
		mov	bp, ax

		clr	cx		; Make es be zero
		mov	es, cx

		dsi			; No interrupts during modification
		mov	ax, [bx]	; Copy vector elements in
		mov	es:[bp], ax
		mov	ax, 2[bx]
		mov	es:2[bp], ax
		eni			; Interrupts now ok

		pop	cx		; Restore trashed registers
		pop	bp
		pop	es

		ret
ResetInterrupt	endp
		bugsym	ResetInterrupt



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		FetchArg
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Fetch an arg from the command tail

CALLED BY:	ScrInit, Main
PASS:		DI	= address of desired arg
		BX	= place to store succeeding word -- 0 if no value
			  for arg.
		DX	= number of bytes in value space.
RETURN:		BX	= address after word (or 1 if arg takes no value)
			  carry set if argument not givien
DESTROYED:	SI, AX, CX, DX

PSEUDO CODE/STRATEGY:
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	9/29/88		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	FetchArg
FetchArg	proc	near
		push	di
		push	bp
		mov	bp, sp
		
		mov	si, 80h 	; Start search from beginning
					; of tail. 
		cld			; Search forward.
		lodsb			; Fetch the tail length
		cbw			; Convert the length
		mov	cx, ax		; Store count in cx
		mov	di, si		; SCAS uses di
FAFindSlash:
		mov	si, di		; needed to support '-' as well as '/'
		; look for either a forward slash or a dash
		; as a flag indicator
		push	cx
		mov	al, '/'	 	; look for /
		repne 	scasb		; do it, babe.
		pop	ax		; get original count in ax
		je	FoundSlash	; Didn't find /
		mov	cx, ax
		mov	al, '-'		; look for -
		mov	di, si		; restore di for second try
		repne	scasb		; do it, honey
		jne	FAFail		; didn't find it,  give up
FoundSlash:
		mov	si, 2[bp]	; Fetch arg name
FACmp:
		lodsb			; Load next arg char
		tst	al		; See if it's null
		jz	FAEqual		; Yes -- consider it a match
		scasb			; Compare it with the next tail char
		jne	FAFindSlash	; Nope -- look for next arg
		loop	FACmp		; Continue the scan unless we hit the
					; end of the tail.
		lodsb			; Might have hit the end of the arg,
					; too. Check for final null
		tst	al
		jne	FAFail		; Nope -- no such arg
		;FALLTHRU
FAEqual:
		tst	bx		; See if arg takes a value
		jnz	FAEqual2	; yes
		inc	bx		; No -- set bx as a flag
		jmp	FACopyDone
FAEqual2:
		mov	al, ':'		; Args are separated from their
					; values by a colon -- find it
		repne	scasb
		jcxz	FAFail		; If hit end of tail, arg invalid
FACopy:
		mov	si, di		; Need tail pointer in si for lodsb
FACopy2:
		lodsb	es:		; Copy bytes in until the end of
					; the tail or we hit a space or slash
		cmp	al, ' '
		je	FACopyDone
		cmp	al, '/'
		je	FACopyDone
		cmp	al, '-'
		je	FACopyDone
		cmp	al, 0dh		; carriage return
		je	FACopyDone
		mov	[bx], al	; Store the value byte
		inc	bx		; Next place in value
		dec	dx		; See if out of room in value
		jz	FACopyDone2	; Yes.
		loop	FACopy2		; Go back as long as there's stuff in
					; the tail.
FACopyDone:
		mov	{byte}[bx], 0	; Null-terminate the value, if there's
				        ; room.
FACopyDone2:
		pop	bp
		pop	di		; Remove di from stack
		ret
FAFail:
		stc
		jmp	FACopyDone2
FetchArg	endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		GetEnv
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Fetch the value of an environment variable

CALLED BY:	GLOBAL
PASS:		DS:SI	= "<var>=" to find
		ES	= PSP segment
RETURN:		ES:DI	= address of value
		CF	= 1 if variable not found, 0 if variable found
DESTROYED:	AX

PSEUDO CODE/STRATEGY:
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	9/29/88		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	GetEnv
GetEnv		proc	near
		cld
		mov	es, es:ENV_SEG

		clr	di
		mov	cx, 0ffffh	; Max allowed
GE1:
		push	si		; Save pattern start
		repe	cmpsb		; Compare the two
		mov	al, -1[si]	; Fetch last pattern byte compared
		pop	si		; Restore pattern start
		tst	al		; See if got to the end of the pattern
		jne	GE2		; No
		dec	di		; Yes. DI points past the first char
					; of the value (the first char was
					; compared against 0 and di advanced),
					; so back it up.
		clc			; Yes. ES:DI points to first char of
					; value. Return success.
		ret
GE2:
		clr	al		; Look for end of this variable. Note
					; that we can't be there already
					; because every variable contains an
					; = that won't match unless it's in the
					; right place.
		repne	scasb
		or	al, es:[di]	; See if at end of whole environment by
					; checking the next byte. If it's 0,
					; we're done.
		jnz	GE1		; More to come
		stc			; Failed -- set carry and return
		ret
GetEnv		endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PCTPrintString
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	print a string to the screen

CALLED BY:	GLOBAL

PASS:		cs:[si] points to a NULL terminated string

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	9/ 8/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
	public PCTPrintString
PCTPrintString	proc	near
	mov	bh, 0		; page zero
	mov	ah, 0eh		; print to screen (teletype mode)
printLoop:
	mov	al, cs:[si]
	tst	al
	jz	done
	inc	si
	int	10h		; zip it off to the screen
	jmp	printLoop 
done:
	ret
PCTPrintString	endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		CheckForExitKey
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	look for any of the exit keys

CALLED BY:	main

PASS:		al = key last read in from keyboard

RETURN:		carry set on exit key, else clear

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	9/ 8/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
	public CheckForExitKey
CheckForExitKey	proc	near
	uses	ax
	.enter
	cmp	al, 'q'
	je	isExit
	cmp	al, 'Q'
	je	isExit
	cmp	al, 3
	je	isExit
	cmp	al, 13
	je	isExit
	clc
done:
	.leave
	ret
isExit:
	stc
	jmp	done
CheckForExitKey	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		WaitForChar
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	wait for a specific character to show up or a NAK

CALLED BY:	GLOBAL

PASS:		al =  char to wait for
		ah =  mask for char  (0ffh for normal use)

RETURN:		carry set if charater not found,
		else ax = NAK or character expected

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	7/31/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
	public	WaitForChar
WaitForChar	proc	near
time	local	dword		; used for timeout purposes
goal	local	byte
mymask	local	byte
	uses	ds, bx
	.enter
	mov	mymask, ah
	mov	goal, al
	GetTimeLocation ds, bx
	mov	ax, ds:[bx]
	mov	time.low, ax
	mov	ax, ds:[bx+2]
	mov	time.high, ax
readLoop:
	push	cs
	pop	ds
	test	sysFlags, MASK SF_EXIT or MASK SF_NUKE_FILE
	jnz	timeOut
	call	ComRead
	jz	checkForTimeout
	and	al, mymask
	cmp	al, goal
	je	gotChar
	cmp	al, NAK
	je	gotChar
	cmp	al, NAK_QUIT
	je	gotChar
checkForTimeout:		
	; if the high words are off by 2 or more, then its been a long time
	; so timeout
	GetTimeLocation	ds, bx		; ds:[bx] = pointer to time location
	mov	ax, ds:[bx+2]	
	sub	ax, time.high
	cmp	ax, 2
	jae	timeOut	

	; otherwise the diffeence is just the subtraction of the low words
	mov	ax, ds:[bx]
	sub	ax, time.low
	cmp	ax, 182		; 182 = 10 seconds
	jae	timeOut
	jmp	readLoop
gotChar:
	clc
done:
	.leave
	ret
timeOut:
	stc
	jmp	done
WaitForChar	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		FileReadBlockHeader
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	wait for a block header charaater to come

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		carry set on timeout, otherwise clear

DESTROYED:	ax

PSEUDOCODE/STRATEGY:	ignore all characters that come in until we get
			a block header, which right now is a single
			character BLOCK_START, and timeout after 10
			seconds if it doesn't come in

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/ 4/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

FileReadBlockHeader	proc	near
	.enter
	mov	al, BLOCK_START
	mov	ah, 0ffh
	call	WaitForChar
if	DEBUG
	pushf
	push	ax
	mov	al, 'H'
	mov	ah, SCREEN_ATTR_INV
	call	DebugPrintChar
	pop	ax
	popf
endif
	jc	done
	cmp	al, BLOCK_START and 7fh
	jne	setCF
	clc
done:	
	.leave
	ret
setCF:
	stc
	jmp	done	
FileReadBlockHeader	endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		FileReadBlockTrailer
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	wait for a block trailer charaater to come

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		carry set on timeout, otherwise clear

DESTROYED:	ax

PSEUDOCODE/STRATEGY:	ignore all characters that come in until we get
			a block header, which right now is a single
			character BLOCK_END, and timeout after 10
			seconds if it doesn't come in

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/ 4/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

FileReadBlockTrailer	proc	near
	.enter
	mov	al, BLOCK_END
	mov	ah, 0ffh
	call	WaitForChar
if	DEBUG
	push	ax
	mov	al, 'T'
	mov	ah, SCREEN_ATTR_INV
	call	DebugPrintChar
	pop	ax
	call	DebugPrintByte
endif
	jc	done
	cmp	al, BLOCK_END and 7fh
	jne	setCF
	clc
done:	
	.leave
	ret
setCF:
	stc
	jmp	done	
FileReadBlockTrailer	endp


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		FileReadBlock
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Read a block of bytes from the serial line

CALLED BY:	FileReceive

PASS: 		DI = Offset of buffer
		ES = Segment of buffer

RETURN:		carry set on timeout, if carry set ax = NAK or NAK_QUIT
		al = last character read
		cx = number of characters actually read
		SF_READ_FILENAME set if first block contained filename,
			which has been copied to the pathname buffer.

DESTROYED:	DI, CX, AX, DF = 0

PSEUDO CODE/STRATEGY:
	Call ComRead until CX goes to 0 or exitFlag is set

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	FileReadBlock
FileReadBlock	proc	near
bufOffset	local	word	; save beginning of buffer pointer also
		uses	bx
		.enter
	
		call	FileReadBlockHeader
		mov	ax, NAK
		jc	done

	; bx will be true only if the last character was a BLOCK_QUOTE
	; if it was then, if the following character is a BLOCK_START
	; or BLOCK_END, then we know that its really just data
		cld

	; set up all the local variables

		mov	bufOffset, di
reset:
		clr	cx
FileReadBlock1:
		call	ComReadWithWait
		jc	timeOut

		cmp	al, BLOCK_QUOTE
		je	gotQuote
		cmp	al, BLOCK_START
		je	doResend
		cmp	al, BLOCK_END
		je	getCRC
justData:
		cmp	cx, BUFFER_SIZE
		je	bogus
		stosb
		inc	cx	; count # bytes actually written
checkFlags::
		test	sysFlags, MASK SF_EXIT or MASK SF_NUKE_FILE
		jnz	doneOK
		jmp	FileReadBlock1
doneOK:
		clc
done:
		.leave
		ret
getCRC:
if	DEBUG
		mov	al, 'C'
		mov	ah, SCREEN_ATTR_INV
		call	DebugPrintChar
endif
	;
	; Read the two bytes of the CRC
	; 
		call	ComReadWithWait
		jc	timeOut
		mov	bl, al
		call	ComReadWithWait
		jc	timeOut
		mov	bh, al

		mov	ax, 0
		mov	di, bufOffset
		call	CalcCRC

if	DEBUG
		pushf
		push	ax
		mov	ax, bx
		call	DebugPrintWord
		pop	ax
		call	DebugPrintWord
		popf
endif

	; 
	; The block has been successfully read in.  If it is the first
	; block, we need to check if it contains a filename, or just data.
	;
		test	sysFlags, mask SF_FIRST_BLOCK
		jz	notFirstBlock
		call	CheckBlockForFilename
		jnc	done			; block contains filename

notFirstBlock:
		cmp	ax, bx
		je	doneOK
		jmp	bogus

		; the followinw character will be a special character that
		; must be translated into data
gotQuote:
		call	ComReadWithWait
		jc	timeOut
		sub	al, BLOCK_START_DATA - BLOCK_START
		jb	bogus
		cmp	al, BLOCK_QUOTE
		jbe	justData
timeOut:
bogus:
		mov	ax, NAK
		stc
		jmp	done
doResend:
		; ok, we got a genuine BLOCK_START so we will drop what we
		; got so far, reset the variables and restart
		mov 	di, bufOffset
		jmp	reset		
FileReadBlock	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		CheckBlockForFilename
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	The first block has been read.  Check if it contains
		the filename instead of data.

CALLED BY:	FileReadBlock
PASS:		es:di - buffer containing the block just read
		ax - calculated CRC
		bx - CRC passed by sender
		cx - block size
RETURN:		carry clear if valid filename block
DESTROYED:	si,di

PSEUDO CODE/STRATEGY:
	If this first block starts with the filenameBlock below, the
	sender is using the new protocol, and filename which follows this
	string should be used instead of the first filename sent.
	The CRC in this case will actually be the CRC + 1.
	
KNOWN BUGS/SIDE EFFECTS/IDEAS:
REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	cassie	12/29/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
;
; filenameBlockString must not change if the PCCom library is to work
; with pcsend.  If it changes here, it must also change in pccomFile.asm
;
filenameBlockString	db	"!PCCom File Transfer Filename Block! "
filenameBlockStringEnd	label	far

CheckBlockForFilename		proc	near
		uses	ax,cx,ds,es
		.enter

	;
	; Is the block large enough to hold the filenameBlock?
	;
		cmp	cx, size filenameBlockString
		jbe	setCF
	
	;
	; Is the filename string the first thing in the block?
	;
		push	cx
		push	cs
		pop	ds
		mov	cx, size filenameBlockString 
		mov	si, offset filenameBlockString
		repe	cmpsb
		pop	cx
		jne	setCF

		inc	ax				; CRC <- CRC + 1
		cmp	ax, bx
		jne	setCF

	;
	; Sender must be using new protocol, because this is a special
	; block which contains the filename.  Save the new filename.
	;

		sub	cx, size filenameBlockString 
		cmp	cx, FILENAME_SIZE
		ja	setCF

		ornf	sysFlags, mask SF_READ_FILENAME
		push	ds, es
		pop	es, ds
		mov	si, di				; ds:si <- source
		mov	di, offset filename		; es:di <- destination
		rep	movsb
		clc	
done:
		.leave
		ret

setCF:
		stc
		jmp	done
CheckBlockForFilename		endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		WriteDataBlock
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	write a block of data adding quotes where neccessary

CALLED BY:	GLOBAL

PASS:		si = start of buffer
		cx = # of bytes to send
	
RETURN:		ax = checksum for block (sum of all the charaters sent)

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/ 6/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

WriteDataBlock	proc	near
		uses	cx, di, bx
		.enter
		cld
		mov	di, si	; save start of buffer for CRC at end
		mov	bx, cx	; save number of bytes, also for CRC
		jcxz	done

if DEBUG
		mov	al, 'W'
		call	DebugPrintChar
endif
		mov	al, BLOCK_START
		call	ComWrite

writeLoop:
		lodsb	; al <- next bytes in buffer
		
		cmp	al, BLOCK_START
		je	writeSpecial
		cmp	al, BLOCK_END
		je	writeSpecial
		cmp	al, BLOCK_QUOTE
		jne	writeChar

writeSpecial:
		mov	ah, al
		mov	al, BLOCK_QUOTE
		call	ComWrite

		mov	al, ah
		.assert (BLOCK_END eq BLOCK_START+1) and \
			   (BLOCK_END_DATA eq BLOCK_START_DATA+1)
		.assert (BLOCK_QUOTE eq BLOCK_END+1) and \
			   (BLOCK_QUOTE_DATA eq BLOCK_END_DATA+1)

		add	al, BLOCK_START_DATA - BLOCK_START
writeChar:
		call	ComWrite
		loop	writeLoop

	; send the END token
		mov	al, BLOCK_END
		call	ComWrite

	; send the block CRC
		mov	cx, bx
		clr	ax
		call	CalcCRC
if DEBUG
		push	ax
		mov	al, 'C'
		call	DebugPrintChar
		pop	ax
		call	DebugPrintWord
endif
		push	ax
		call	ComWrite
		mov	al, ah
		call	ComWrite
		pop	ax
done:
		.leave
		ret
WriteDataBlock	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		CalcCRC
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Calculate 16-bit CRC

CALLED BY:	FileReadBlock, WriteDataBlock

PASS:		es:di		- packet of chars to calculate CRC for
		ax		- initial value (usually 0)
		cx		- number of chars in packet
		ds		- dgroup

RETURN:		ax		- CRC

DESTROYED:	none

PSEUDO CODE/STRATEGY:


KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	dennis	12/22/89	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

crcTable	label	word
dw	0x0000,  0x1021,  0x2042,  0x3063,  0x4084,  0x50a5,  0x60c6,  0x70e7
dw	0x8108,  0x9129,  0xa14a,  0xb16b,  0xc18c,  0xd1ad,  0xe1ce,  0xf1ef
dw	0x1231,  0x0210,  0x3273,  0x2252,  0x52b5,  0x4294,  0x72f7,  0x62d6
dw	0x9339,  0x8318,  0xb37b,  0xa35a,  0xd3bd,  0xc39c,  0xf3ff,  0xe3de
dw	0x2462,  0x3443,  0x0420,  0x1401,  0x64e6,  0x74c7,  0x44a4,  0x5485
dw	0xa56a,  0xb54b,  0x8528,  0x9509,  0xe5ee,  0xf5cf,  0xc5ac,  0xd58d
dw	0x3653,  0x2672,  0x1611,  0x0630,  0x76d7,  0x66f6,  0x5695,  0x46b4
dw	0xb75b,  0xa77a,  0x9719,  0x8738,  0xf7df,  0xe7fe,  0xd79d,  0xc7bc
dw	0x48c4,  0x58e5,  0x6886,  0x78a7,  0x0840,  0x1861,  0x2802,  0x3823
dw	0xc9cc,  0xd9ed,  0xe98e,  0xf9af,  0x8948,  0x9969,  0xa90a,  0xb92b
dw	0x5af5,  0x4ad4,  0x7ab7,  0x6a96,  0x1a71,  0x0a50,  0x3a33,  0x2a12
dw	0xdbfd,  0xcbdc,  0xfbbf,  0xeb9e,  0x9b79,  0x8b58,  0xbb3b,  0xab1a
dw	0x6ca6,  0x7c87,  0x4ce4,  0x5cc5,  0x2c22,  0x3c03,  0x0c60,  0x1c41
dw	0xedae,  0xfd8f,  0xcdec,  0xddcd,  0xad2a,  0xbd0b,  0x8d68,  0x9d49
dw	0x7e97,  0x6eb6,  0x5ed5,  0x4ef4,  0x3e13,  0x2e32,  0x1e51,  0x0e70
dw	0xff9f,  0xefbe,  0xdfdd,  0xcffc,  0xbf1b,  0xaf3a,  0x9f59,  0x8f78
dw	0x9188,  0x81a9,  0xb1ca,  0xa1eb,  0xd10c,  0xc12d,  0xf14e,  0xe16f
dw	0x1080,  0x00a1,  0x30c2,  0x20e3,  0x5004,  0x4025,  0x7046,  0x6067
dw	0x83b9,  0x9398,  0xa3fb,  0xb3da,  0xc33d,  0xd31c,  0xe37f,  0xf35e
dw	0x02b1,  0x1290,  0x22f3,  0x32d2,  0x4235,  0x5214,  0x6277,  0x7256
dw	0xb5ea,  0xa5cb,  0x95a8,  0x8589,  0xf56e,  0xe54f,  0xd52c,  0xc50d
dw	0x34e2,  0x24c3,  0x14a0,  0x0481,  0x7466,  0x6447,  0x5424,  0x4405
dw	0xa7db,  0xb7fa,  0x8799,  0x97b8,  0xe75f,  0xf77e,  0xc71d,  0xd73c
dw	0x26d3,  0x36f2,  0x0691,  0x16b0,  0x6657,  0x7676,  0x4615,  0x5634
dw	0xd94c,  0xc96d,  0xf90e,  0xe92f,  0x99c8,  0x89e9,  0xb98a,  0xa9ab
dw	0x5844,  0x4865,  0x7806,  0x6827,  0x18c0,  0x08e1,  0x3882,  0x28a3
dw	0xcb7d,  0xdb5c,  0xeb3f,  0xfb1e,  0x8bf9,  0x9bd8,  0xabbb,  0xbb9a
dw	0x4a75,  0x5a54,  0x6a37,  0x7a16,  0x0af1,  0x1ad0,  0x2ab3,  0x3a92
dw	0xfd2e,  0xed0f,  0xdd6c,  0xcd4d,  0xbdaa,  0xad8b,  0x9de8,  0x8dc9
dw	0x7c26,  0x6c07,  0x5c64,  0x4c45,  0x3ca2,  0x2c83,  0x1ce0,  0x0cc1
dw	0xef1f,  0xff3e,  0xcf5d,  0xdf7c,  0xaf9b,  0xbfba,  0x8fd9,  0x9ff8
dw	0x6e17,  0x7e36,  0x4e55,  0x5e74,  0x2e93,  0x3eb2,  0x0ed1,  0x1ef0

CalcCRC	proc	near
	uses	bx,cx,di
	.enter
doCRC:
	mov	bx, ax				;		
	xor	bh, es:[di]			; (chksm>>8)^c]
	inc	di				; advance char ptr
	mov	bl, bh				;point to table
	clr	bh
	shl	bx, 1				;compute offset into table
	mov	ah, al 				;
	clr	al				;chksm<<8
	xor	ax, cs:crcTable[bx]		;chksm<<8 ^ crctab[(chksm>>8)^c]
	loop	doCRC
	.leave
	ret
CalcCRC	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		IncCRC
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	increment the CRC

CALLED BY:	GLOBAL

PASS:		ax = old crc
		bl = next character
		
RETURN:		ax = new crc

DESTROYED:	none

PSEUDOCODE/STRATEGY:	
	short
	IncCRC(short crc, char c)
	{ 
	    unsigned short   cs;
	    char   *ind;
	    short   val;

	    cs = crc;
	    cs ^= (short)(c << 8);
	    ind = (char *)crcTable;
	    cs = (cs >> 8);
	    cs = (cs << 1);
	    ind += cs;
	    val = *(short *)ind;
	    crc = (crc << 8);
	    crc ^= val;
	    return crc;
	}

	or

	    return ((crc << 8) ^ crcTable[((crc >> 8) ^ c) & 0xff]);

	for short.

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/12/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

IncCRC	proc	near
	uses	bx
	.enter
	xor	ah, bl				; crc.high ^= char
	xchg	al, ah				; crc <<= 8
	mov	bx, ax				; bx <- crcTable index
	clr	bh
	mov	al, bh				; finish <<= 8...
	shl	bx				; table holds words...
	xor	ax, cs:[crcTable][bx]		; crc = (crc<<8) ^
						;  crcTable[(crc>>8)^c]
	.leave
	ret
IncCRC	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		Sync
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	sync up by waiting for a ffh

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		carry set if not found, else clear

DESTROYED:	ax

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	7/31/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

Sync	proc	near
	mov	al, 7fh
	mov	ah, 7fh
	call	WaitForChar
	ret
Sync	endp


if	DEBUG


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		DebugPrintString
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Print a null-terminated string

CALLED BY:	Debugging things
PASS:		ds:si	= null-terminated string to print
		ah	= attribute
RETURN:		nothing
DESTROYED:	

PSEUDO CODE/STRATEGY:
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	1/13/92		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
DebugPrintString proc	near
		uses	al, si
		.enter
printLoop:
		lodsb
		tst	al
		jz	done
		call	DebugPrintChar
		jmp	printLoop
done:
		.leave
		ret
DebugPrintString endp

;nibbles		db	"0123456789ABCDEF"

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		DebugPrintByte
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Print a byte in HEX on the screen

CALLED BY:	Debugging things

PASS:		AL	= byte to print

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/27/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
DebugPrintByte	proc	near
		pushf
		push	bx
		push	ax
		mov	bx, offset nibbles
		shr	al, 1
		shr	al, 1
		shr	al, 1
		shr	al, 1
		and	al, 0fh
		xlatb	cs:
		mov	ah, SCREEN_ATTR_NORMAL
		call	DebugPrintChar
		pop	ax
		push	ax
		and	al, 0fh
		xlatb	cs:
		mov	ah, SCREEN_ATTR_NORMAL
		call	DebugPrintChar

		mov	ax, (SCREEN_ATTR_NORMAL shl 8) or ' '
		call	DebugPrintChar
		pop	ax
		pop	bx
		popf
		ret
DebugPrintByte	endp

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		DebugPrintWord
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Print a word in HEX on the screen

CALLED BY:	Debugging things

PASS:		AX	= byte to print

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/27/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
DebugPrintWord	proc	near
		uses	ax, bx, cx
		.enter
		pushf
		mov	bx, offset nibbles
		mov	cl, 4
		xchg	al, ah
		ror	al, cl
		call	printNibble
		ror	al, cl
		call	printNibble
		xchg	al, ah
		ror	al, cl
		call	printNibble
		ror	al, cl
		call	printNibble

		mov	ax, (SCREEN_ATTR_NORMAL shl 8) or ' '
		call	DebugPrintChar
		popf
		.leave
		ret

printNibble:
		push	ax
		and	al, 0fh
		xlatb	cs:
		mov	ah, SCREEN_ATTR_NORMAL
		call	DebugPrintChar
		pop	ax
		retn
DebugPrintWord	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		DebugPrintChar
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Print the passed character to the screen, obeying the
		attribute request, if possible.

CALLED BY:	Debugging things
PASS:		ah	= attribute
		al	= character
RETURN:		nothing
DESTROYED:	nothing

PSEUDO CODE/STRATEGY:
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	7/12/91		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
 if DEBUG_USE_BIOS
DebugPrintChar	proc	near
		uses	ax
		.enter
		mov	ah, 0xe
		int	10h
		.leave
		ret
DebugPrintChar	endp
 else


MonoScreen segment at SCREEN_SEG
screenBase	label	word
MonoScreen ends

curPos		fptr.word	screenBase; current location in mono screen

DebugPrintChar	proc	near
		uses	es, di
		.enter
		pushf
		les	di, cs:[curPos]
		stosw
		cmp	di, SCREEN_SIZE
		jb	setCursor
		clr	di
setCursor:
		mov	{word}es:[di], ((SCREEN_ATTR_NORMAL or 0x80) shl 8) or \
					0x7f
		mov	cs:[curPos].offset, di
		popf
		.leave
		ret
DebugPrintChar	endp
 endif
endif
program		ends
