COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PROJECT:	GEOS
MODULE:		pcsend, pcget
FILE:		main.asm

AUTHOR:		Adam de Boor

ROUTINES:
	Name		Description
	----		-----------
	
REVISION HISTORY:
	Name	Date	Description
	----	----	-----------
	adam	6/7	Initial Revision


DESCRIPTION:

	Initialization code common to pcsend and pcget.

	$Id: main.asm,v 1.10 94/01/22 14:44:14 cassie Exp $
	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
MAINMODULE	=	1
		include	com43.def

program		segment 
global	ComRawWrite:far
global  DebugPrintChar:near
global  ComDrainQueue:near

checksum	dw	0
sysFlags	SysFlags	<>
nibbles             db      "0123456789ABCDEF"

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		MainCommon
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Entry point for Com43

CALLED BY:	Main (pcsend, pcget)

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	Everything

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

MainCommon	proc 	near
		push	ds
		pop	es
		call	CheckIsPC	; check for running on a pc.
					; Sets 'isBasePC'.
		call	ComInit
		jc	done
		call	GetDirectory
		clc
done:
		ret
MainCommon		endp

DIR_LENGTH	equ	80

dirFlag		byte 	'd', 0
directory	byte    DIR_LENGTH	dup(0)

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		GetDirectory
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	get the directory to work in from the command line
		(the default is the current directory)

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	9/ 2/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
GetDirectory	proc	far

	mov	di, offset dirFlag
	mov	bx, offset directory
	mov	{byte}ds:[bx], 0		; clear directory
	mov	dx, DIR_LENGTH
	call	FetchArg
	ret
GetDirectory	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		SkipWhiteSpace
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	skip the white space (actually just spaces...)

CALLED BY:	GLOBAL

PASS:		ds:si = string in whic to skip the white space

RETURN:		al = first non-space character in string, 
		ds:si: points at character following the 
		       first non-space character

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/31/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

SkipWhiteSpace	proc	near
	.enter
skipLoop:
	lodsb
	cmp	al, ' '
	je	skipLoop
	.leave
	ret
SkipWhiteSpace	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		GetNextFileName
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	get the next file to send

CALLED BY:	main

PASS:		ds:80h has the command line options passed in

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/31/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
filename	byte  	DOS_FILENAME_LENGTH+1	dup(?)

GetNextFileName	proc	near
		.enter
		mov	si, 80h 	; Start search from beginning
					; of tail. 
		cld			; Search forward.
		lodsb			; Fetch the tail length
		cbw			; Convert the length
		mov	cx, ax
		add	cx, si
		; now go looking through the arguments for the first
		; one that doesn't start with a / as it must be
		; a filename, after I get the filename, overwrite the
		; first letter of the filename with a / so the next time
		; we get the next one, we won't be needing the old one again
		; anyways 
searchLoop:
		cmp	si, cx
		jge	noMoreFiles
		call	SkipWhiteSpace
		cmp	al, '/'
		jne	foundName
skipArgLoop:
		lodsb
		cmp	al, ' '
		je	searchLoop
		cmp	si, cx
		jge	noMoreFiles
		jmp	skipArgLoop
foundName:
		mov	di, offset filename
		mov	{byte}es:[di], al
		inc	di	
		dec	si
		mov	{byte}ds:[si], '/' 
		inc	si
copyLoop:
		; get the filename, which can be ternimated by any of the
		; following characters: space, carriage return, / or end
		; of the tail
		lodsb
		cmp	al, ' '			; check for space
		je	foundFile
		cmp	al, 0dh			; check for carriage return
		je	foundFile
		cmp	al, '/'			; check for slash
		je	foundFile
		mov	es:[di], al
		inc	di
		cmp	si, cx			; check for end of the tail
		jl	copyLoop
foundFile:
		mov	{byte}es:[di], '\0'
		clc
done:			
		.leave
		ret
noMoreFiles:
		stc
		jmp	done
GetNextFileName	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		FileOpenError
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	put out a message for a file open error to the screen

CALLED BY:	GLOBAL

PASS:		al = error code

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/ 3/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
FileOpenErrors	nptr \
	offset FOFunctionNumberInvalid,
	offset FOFileNotFound,		
	offset FOPathNotFound,		
	offset FONoHandleAvailable,
	offset FOAccessDenied,		
	offset FOAccessCodeInvalid	
	
FOFunctionNumberInvalid	byte	"Function Number Invalid",'\r','\n',0
FOFileNotFound		byte	"File not found",'\r','\n',0
FOPathNotFound		byte	"Path Not Found",'\r','\n',0
FONoHandleAvailable	byte	"No File Handles Available",'\r','\n',0
FOAccessDenied		byte	"Access Denied",'\r','\n',0
FOAccessCodeInvalid	byte	"Access Code Invaild",'\r','\n',0

FileOpenError	proc	near
	.enter
	cmp	al, 5
	jle	contFileError
	mov	al, 6
contFileError:
	mov	si, offset FileOpenErrors
	dec	al
	shl	al
	clr	ah
	add	si, ax
	mov	si, cs:[si]
	call	PCTPrintString
	.leave
	ret
FileOpenError	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		FileReadError
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	output a message on a file read error to the screen

CALLED BY:	GLOBAL

PASS:		ax = error code

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/ 3/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
FileReadErrors	nptr \
	offset FOAccessDenied,
	offset FRInvalidHandle,
	offset DirectoryNotFound

FRInvalidHandle		byte	"Invalid Handle",'\r','\n',0

FileReadErrorString	byte	"File Read Error: ",'\r','\n',0

DirectoryNotFound	byte	"Directory not found.",'\r', '\n',0

FileReadError	proc	near
	.enter
	sub	al, 5
	mov	si, offset FileReadErrors
	shl	al
	clr	ah
	add	si, ax
	mov	si, cs:[si]
	call	PCTPrintString
	.leave
	ret
FileReadError	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		FileCreateError
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	output a message on a file create error to the screen

CALLED BY:	GLOBAL

PASS:		al = error code

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/ 3/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

FileCreateErrors 	nptr\
	offset	FCPathNotFound,
	offset	FCNoHandleAvailable,
	offset	FOAccessDenied

FCPathNotFound		byte	"Path Not Found", '\r', '\n',0
FCNoHandleAvailable	byte	"No Handle Available", '\r','\n',0

FileCreateError	proc	near
	.enter
	sub	al, 3
	mov	si, offset FileCreateErrors
	dec	al
	shl	al
	clr	ah
	add	si, ax
	mov	si, cs:[si]
	call	PCTPrintString
	.leave
	ret
FileCreateError	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		Abort
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	put up a timeout message

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/ 3/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

TimeoutString	byte	"Timed Out: Transfer Aborted",'\r','\n',0

Abort	proc	near
	.enter
	mov	si, offset TimeoutString
	call	PCTPrintString
	.leave
	ret
Abort	endp



program		ends
		end	Main
